# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
ShapeProcess module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_shapeprocess.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ShapeProcess
else:
    import _ShapeProcess

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _ShapeProcess.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _ShapeProcess.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _ShapeProcess.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _ShapeProcess.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _ShapeProcess.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _ShapeProcess.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _ShapeProcess.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _ShapeProcess.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _ShapeProcess.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _ShapeProcess.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _ShapeProcess.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _ShapeProcess.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _ShapeProcess.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _ShapeProcess.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _ShapeProcess.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _ShapeProcess.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _ShapeProcess.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _ShapeProcess:
_ShapeProcess.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _ShapeProcess.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.Resource
import OCC.Core.TopoDS
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp
import OCC.Core.BRepTools
import OCC.Core.Bnd
import OCC.Core.TColgp
import OCC.Core.BVH
import OCC.Core.Geom
import OCC.Core.GeomAbs
import OCC.Core.Geom2d
import OCC.Core.TopTools
import OCC.Core.BRep
import OCC.Core.Poly
import OCC.Core.TShort
import OCC.Core.ShapeExtend
import OCC.Core.TColGeom
import OCC.Core.ShapeBuild

from enum import IntEnum
from OCC.Core.Exception import *




def Handle_ShapeProcess_Context_Create() -> "opencascade::handle< ShapeProcess_Context >":
    return _ShapeProcess.Handle_ShapeProcess_Context_Create()

def Handle_ShapeProcess_Context_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeProcess_Context >":
    return _ShapeProcess.Handle_ShapeProcess_Context_DownCast(t)

def Handle_ShapeProcess_Context_IsNull(t: "opencascade::handle< ShapeProcess_Context > const &") -> "bool":
    return _ShapeProcess.Handle_ShapeProcess_Context_IsNull(t)

def Handle_ShapeProcess_Operator_Create() -> "opencascade::handle< ShapeProcess_Operator >":
    return _ShapeProcess.Handle_ShapeProcess_Operator_Create()

def Handle_ShapeProcess_Operator_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeProcess_Operator >":
    return _ShapeProcess.Handle_ShapeProcess_Operator_DownCast(t)

def Handle_ShapeProcess_Operator_IsNull(t: "opencascade::handle< ShapeProcess_Operator > const &") -> "bool":
    return _ShapeProcess.Handle_ShapeProcess_Operator_IsNull(t)

def Handle_ShapeProcess_ShapeContext_Create() -> "opencascade::handle< ShapeProcess_ShapeContext >":
    return _ShapeProcess.Handle_ShapeProcess_ShapeContext_Create()

def Handle_ShapeProcess_ShapeContext_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeProcess_ShapeContext >":
    return _ShapeProcess.Handle_ShapeProcess_ShapeContext_DownCast(t)

def Handle_ShapeProcess_ShapeContext_IsNull(t: "opencascade::handle< ShapeProcess_ShapeContext > const &") -> "bool":
    return _ShapeProcess.Handle_ShapeProcess_ShapeContext_IsNull(t)

def Handle_ShapeProcess_UOperator_Create() -> "opencascade::handle< ShapeProcess_UOperator >":
    return _ShapeProcess.Handle_ShapeProcess_UOperator_Create()

def Handle_ShapeProcess_UOperator_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< ShapeProcess_UOperator >":
    return _ShapeProcess.Handle_ShapeProcess_UOperator_DownCast(t)

def Handle_ShapeProcess_UOperator_IsNull(t: "opencascade::handle< ShapeProcess_UOperator > const &") -> "bool":
    return _ShapeProcess.Handle_ShapeProcess_UOperator_IsNull(t)
class shapeprocess(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def FindOperator(*args) -> "Standard_Boolean":
        r"""
        Finds operator by its name.

        Parameters
        ----------
        name: char *
        op: ShapeProcess_Operator

        Returns
        -------
        bool

        """
        return _ShapeProcess.shapeprocess_FindOperator(*args)

    @staticmethod
    def Perform(*args) -> "Standard_Boolean":
        r"""
        Performs a specified sequence of operators on context resource file and other data should be already loaded to context (including description of sequence seq).

        Parameters
        ----------
        context: ShapeProcess_Context
        seq: char *
        theProgress: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        bool

        """
        return _ShapeProcess.shapeprocess_Perform(*args)

    @staticmethod
    def RegisterOperator(*args) -> "Standard_Boolean":
        r"""
        Registers operator to make it visible for performer.

        Parameters
        ----------
        name: char *
        op: ShapeProcess_Operator

        Returns
        -------
        bool

        """
        return _ShapeProcess.shapeprocess_RegisterOperator(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _ShapeProcess.shapeprocess_swiginit(self, _ShapeProcess.new_shapeprocess())
    __swig_destroy__ = _ShapeProcess.delete_shapeprocess

# Register shapeprocess in _ShapeProcess:
_ShapeProcess.shapeprocess_swigregister(shapeprocess)

def shapeprocess_FindOperator(*args) -> "Standard_Boolean":
    r"""
    Finds operator by its name.

    Parameters
    ----------
    name: char *
    op: ShapeProcess_Operator

    Returns
    -------
    bool

    """
    return _ShapeProcess.shapeprocess_FindOperator(*args)

def shapeprocess_Perform(*args) -> "Standard_Boolean":
    r"""
    Performs a specified sequence of operators on context resource file and other data should be already loaded to context (including description of sequence seq).

    Parameters
    ----------
    context: ShapeProcess_Context
    seq: char *
    theProgress: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    bool

    """
    return _ShapeProcess.shapeprocess_Perform(*args)

def shapeprocess_RegisterOperator(*args) -> "Standard_Boolean":
    r"""
    Registers operator to make it visible for performer.

    Parameters
    ----------
    name: char *
    op: ShapeProcess_Operator

    Returns
    -------
    bool

    """
    return _ShapeProcess.shapeprocess_RegisterOperator(*args)

class ShapeProcess_Context(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty tool.

        Returns
        -------
        None
        Creates a new tool and initialises by name of resource file and (if specified) starting scope calls method init().

        Parameters
        ----------
        file: char *
        scope: char *,optional
        	default value is 

        Returns
        -------
        None

        """
        _ShapeProcess.ShapeProcess_Context_swiginit(self, _ShapeProcess.new_ShapeProcess_Context(*args))

    def BooleanVal(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        param: char *
        def: bool

        Returns
        -------
        bool

        """
        return _ShapeProcess.ShapeProcess_Context_BooleanVal(self, *args)

    def GetBoolean(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        param: char *

        Returns
        -------
        val: bool

        """
        return _ShapeProcess.ShapeProcess_Context_GetBoolean(self, *args)

    def GetInteger(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        param: char *

        Returns
        -------
        val: int

        """
        return _ShapeProcess.ShapeProcess_Context_GetInteger(self, *args)

    def GetReal(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        param: char *

        Returns
        -------
        val: float

        """
        return _ShapeProcess.ShapeProcess_Context_GetReal(self, *args)

    def GetString(self, *args) -> "Standard_Boolean":
        r"""
        Get value of parameter as being of specific type returns false if parameter is not defined or has a wrong type.

        Parameters
        ----------
        param: char *
        val: TCollection_AsciiString

        Returns
        -------
        bool

        """
        return _ShapeProcess.ShapeProcess_Context_GetString(self, *args)

    def Init(self, *args) -> "Standard_Boolean":
        r"""
        Initialises a tool by loading resource file and (if specified) sets starting scope returns false if resource file not found.

        Parameters
        ----------
        file: char *
        scope: char *,optional
        	default value is 

        Returns
        -------
        bool

        """
        return _ShapeProcess.ShapeProcess_Context_Init(self, *args)

    def IntegerVal(self, *args) -> "Standard_Integer":
        r"""
        No available documentation.

        Parameters
        ----------
        param: char *
        def: int

        Returns
        -------
        int

        """
        return _ShapeProcess.ShapeProcess_Context_IntegerVal(self, *args)

    def IsParamSet(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if parameter is defined in the resource file.

        Parameters
        ----------
        param: char *

        Returns
        -------
        bool

        """
        return _ShapeProcess.ShapeProcess_Context_IsParamSet(self, *args)

    def LoadResourceManager(self, *args) -> "opencascade::handle< Resource_Manager >":
        r"""
        Loading resource_manager object if this object not equal internal static resource_manager object or internal static resource_manager object is null.

        Parameters
        ----------
        file: char *

        Returns
        -------
        opencascade::handle<Resource_Manager>

        """
        return _ShapeProcess.ShapeProcess_Context_LoadResourceManager(self, *args)

    def Messenger(self, *args) -> "opencascade::handle< Message_Messenger >":
        r"""
        Returns messenger used for outputting messages.

        Returns
        -------
        opencascade::handle<Message_Messenger>

        """
        return _ShapeProcess.ShapeProcess_Context_Messenger(self, *args)

    def RealVal(self, *args) -> "Standard_Real":
        r"""
        No available documentation.

        Parameters
        ----------
        param: char *
        def: float

        Returns
        -------
        float

        """
        return _ShapeProcess.ShapeProcess_Context_RealVal(self, *args)

    def ResourceManager(self, *args) -> "opencascade::handle< Resource_Manager > const &":
        r"""
        Returns internal resource_manager object.

        Returns
        -------
        opencascade::handle<Resource_Manager>

        """
        return _ShapeProcess.ShapeProcess_Context_ResourceManager(self, *args)

    def SetMessenger(self, *args) -> "void":
        r"""
        Sets messenger used for outputting messages.

        Parameters
        ----------
        messenger: Message_Messenger

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_Context_SetMessenger(self, *args)

    def SetScope(self, *args) -> "void":
        r"""
        Set a new (sub)scope.

        Parameters
        ----------
        scope: char *

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_Context_SetScope(self, *args)

    def SetTraceLevel(self, *args) -> "void":
        r"""
        Sets trace level used for outputting messages - 0: no trace at all - 1: errors - 2: errors and warnings - 3: all messages default is 1 : errors traced.

        Parameters
        ----------
        tracelev: int

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_Context_SetTraceLevel(self, *args)

    def StringVal(self, *args) -> "char const *":
        r"""
        Get value of parameter as being of specific type if parameter is not defined or does not have expected type, returns default value as specified.

        Parameters
        ----------
        param: char *
        def: char *

        Returns
        -------
        char *

        """
        return _ShapeProcess.ShapeProcess_Context_StringVal(self, *args)

    def TraceLevel(self, *args) -> "Standard_Integer":
        r"""
        Returns trace level used for outputting messages.

        Returns
        -------
        int

        """
        return _ShapeProcess.ShapeProcess_Context_TraceLevel(self, *args)

    def UnSetScope(self, *args) -> "void":
        r"""
        Go out of current scope.

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_Context_UnSetScope(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeProcess_Context_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeProcess.delete_ShapeProcess_Context

# Register ShapeProcess_Context in _ShapeProcess:
_ShapeProcess.ShapeProcess_Context_swigregister(ShapeProcess_Context)

class ShapeProcess_OperLibrary(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def ApplyModifier(*args) -> "TopoDS_Shape":
        r"""
        Applies breptools_modification to a shape, taking into account sharing of components of compounds. if themutableinput vat is set to true then imput shape s can be modified during the modification process.

        Parameters
        ----------
        S: TopoDS_Shape
        context: ShapeProcess_ShapeContext
        M: BRepTools_Modification
        map: TopTools_DataMapOfShapeShape
        msg: ShapeExtend_MsgRegistrator,optional
        	default value is 0
        theMutableInput: bool,optional
        	default value is Standard_False

        Returns
        -------
        TopoDS_Shape

        """
        return _ShapeProcess.ShapeProcess_OperLibrary_ApplyModifier(*args)

    @staticmethod
    def Init(*args) -> "void":
        r"""
        Registers all the operators.

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_OperLibrary_Init(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _ShapeProcess.ShapeProcess_OperLibrary_swiginit(self, _ShapeProcess.new_ShapeProcess_OperLibrary())
    __swig_destroy__ = _ShapeProcess.delete_ShapeProcess_OperLibrary

# Register ShapeProcess_OperLibrary in _ShapeProcess:
_ShapeProcess.ShapeProcess_OperLibrary_swigregister(ShapeProcess_OperLibrary)

def ShapeProcess_OperLibrary_ApplyModifier(*args) -> "TopoDS_Shape":
    r"""
    Applies breptools_modification to a shape, taking into account sharing of components of compounds. if themutableinput vat is set to true then imput shape s can be modified during the modification process.

    Parameters
    ----------
    S: TopoDS_Shape
    context: ShapeProcess_ShapeContext
    M: BRepTools_Modification
    map: TopTools_DataMapOfShapeShape
    msg: ShapeExtend_MsgRegistrator,optional
    	default value is 0
    theMutableInput: bool,optional
    	default value is Standard_False

    Returns
    -------
    TopoDS_Shape

    """
    return _ShapeProcess.ShapeProcess_OperLibrary_ApplyModifier(*args)

def ShapeProcess_OperLibrary_Init(*args) -> "void":
    r"""
    Registers all the operators.

    Returns
    -------
    None

    """
    return _ShapeProcess.ShapeProcess_OperLibrary_Init(*args)

class ShapeProcess_Operator(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def Perform(self, *args) -> "Standard_Boolean":
        r"""
        Performs operation and eventually records changes in the context.

        Parameters
        ----------
        context: ShapeProcess_Context
        theProgress: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        bool

        """
        return _ShapeProcess.ShapeProcess_Operator_Perform(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeProcess_Operator_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeProcess.delete_ShapeProcess_Operator

# Register ShapeProcess_Operator in _ShapeProcess:
_ShapeProcess.ShapeProcess_Operator_swigregister(ShapeProcess_Operator)

class ShapeProcess_ShapeContext(ShapeProcess_Context):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        file: char *
        seq: char *,optional
        	default value is 

        Returns
        -------
        None
        Initializes a tool by resource file and shape to be processed.

        Parameters
        ----------
        S: TopoDS_Shape
        file: char *
        seq: char *,optional
        	default value is 

        Returns
        -------
        None

        """
        _ShapeProcess.ShapeProcess_ShapeContext_swiginit(self, _ShapeProcess.new_ShapeProcess_ShapeContext(*args))

    def AddMessage(self, *args) -> "void":
        r"""
        Record a message for shape s shape s should be one of subshapes of original shape (or whole one), but not one of intermediate shapes records only if message() is not null.

        Parameters
        ----------
        S: TopoDS_Shape
        msg: Message_Msg
        gravity: Message_Gravity,optional
        	default value is Message_Warning

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_AddMessage(self, *args)

    def ContinuityVal(self, *args) -> "GeomAbs_Shape":
        r"""
        Get value of parameter as being of the type geomabs_shape if parameter is not defined or does not have expected type, returns default value as specified.

        Parameters
        ----------
        param: char *
        def: GeomAbs_Shape

        Returns
        -------
        GeomAbs_Shape

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_ContinuityVal(self, *args)

    def GetContinuity(self, *args) -> "Standard_Boolean":
        r"""
        Get value of parameter as being of the type geomabs_shape returns false if parameter is not defined or has a wrong type.

        Parameters
        ----------
        param: char *
        val: GeomAbs_Shape

        Returns
        -------
        bool

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_GetContinuity(self, *args)

    def GetDetalisation(self, *args) -> "TopAbs_ShapeEnum":
        r"""
        Set and get value for detalisation level only shapes of types from topods_compound and until specified detalisation level will be recorded in maps to cancel mapping, use topabs_shape to force full mapping, use topabs_vertex the default level is topabs_face.

        Returns
        -------
        TopAbs_ShapeEnum

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_GetDetalisation(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Initializes tool by a new shape and clears all results.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_Init(self, *args)

    def IsNonManifold(self, *args) -> "Standard_Boolean":
        r"""
        Get nonmanifold flag.

        Returns
        -------
        bool

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_IsNonManifold(self, *args)

    def Map(self, *args) -> "TopTools_DataMapOfShapeShape const &":
        r"""
        Returns map of replacements shape -> shape this map is not recursive.

        Returns
        -------
        TopTools_DataMapOfShapeShape

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_Map(self, *args)

    def Messages(self, *args) -> "opencascade::handle< ShapeExtend_MsgRegistrator > const &":
        r"""
        No available documentation.

        Returns
        -------
        opencascade::handle<ShapeExtend_MsgRegistrator>

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_Messages(self, *args)

    def PrintStatistics(self, *args) -> "void":
        r"""
        Prints statistics on shape processing onto the current messenger.

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_PrintStatistics(self, *args)

    def RecordModification(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        repl: TopTools_DataMapOfShapeShape
        msg: ShapeExtend_MsgRegistrator,optional
        	default value is 0

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        repl: ShapeBuild_ReShape
        msg: ShapeExtend_MsgRegistrator

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        repl: ShapeBuild_ReShape

        Returns
        -------
        None
        Records modifications and resets result accordingly note: modification of resulting shape should be explicitly defined in the maps along with modifications of subshapes //! in the last function, sh is the shape on which modifier was run. it can be different from the whole shape, but in that case result as a whole should be reset later either by call to setresult(), or by another call to recordmodification() which contains mapping of current result to a new one explicitly.

        Parameters
        ----------
        sh: TopoDS_Shape
        repl: BRepTools_Modifier
        msg: ShapeExtend_MsgRegistrator,optional
        	default value is 0

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_RecordModification(self, *args)

    def Result(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns current result.

        Returns
        -------
        TopoDS_Shape

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_Result(self, *args)

    def SetDetalisation(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        level: TopAbs_ShapeEnum

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_SetDetalisation(self, *args)

    def SetNonManifold(self, *args) -> "void":
        r"""
        Set nonmanifold flag.

        Parameters
        ----------
        theNonManifold: bool

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_SetNonManifold(self, *args)

    def SetResult(self, *args) -> "void":
        r"""
        Sets a new result shape note: this method should be used very carefully to keep consistency of modifications it is recommended to use recordmodification() methods with explicit definition of mapping from current result to a new one.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_SetResult(self, *args)

    def Shape(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns shape being processed.

        Returns
        -------
        TopoDS_Shape

        """
        return _ShapeProcess.ShapeProcess_ShapeContext_Shape(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_ShapeProcess_ShapeContext_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeProcess.delete_ShapeProcess_ShapeContext

# Register ShapeProcess_ShapeContext in _ShapeProcess:
_ShapeProcess.ShapeProcess_ShapeContext_swigregister(ShapeProcess_ShapeContext)

class ShapeProcess_UOperator(ShapeProcess_Operator):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates operator with implementation defined as operfunc (static function).

        Parameters
        ----------
        func: ShapeProcess_OperFunc

        Returns
        -------
        None

        """
        _ShapeProcess.ShapeProcess_UOperator_swiginit(self, _ShapeProcess.new_ShapeProcess_UOperator(*args))


    @staticmethod
    def DownCast(t):
      return Handle_ShapeProcess_UOperator_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _ShapeProcess.delete_ShapeProcess_UOperator

# Register ShapeProcess_UOperator in _ShapeProcess:
_ShapeProcess.ShapeProcess_UOperator_swigregister(ShapeProcess_UOperator)








#!/usr/bin/env python

# Copyright 2009-2016 Thomas Paviot (tpaviot@gmail.com)
##
# This file is part of pythonOCC.
##
# pythonOCC is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# pythonOCC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with pythonOCC.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os
import sys
from typing import Any, Callable, List, Optional, Tuple

from OCC import VERSION
from OCC.Display.backend import load_backend, get_qt_modules
from OCC.Display.OCCViewer import OffscreenRenderer

log = logging.getLogger(__name__)


def check_callable(_callable: Callable) -> None:
    if not callable(_callable):
        raise AssertionError("The function supplied is not callable")


def init_display(backend_str: Optional[str]=None,
                 size: Optional[Tuple[int, int]]=(1024, 768),
                 display_triedron: Optional[bool]=True,
                 background_gradient_color1: Optional[List[int]]=[206, 215, 222],
                 background_gradient_color2: Optional[List[int]]=[128, 128, 128]):
    """ This function loads and initialize a GUI using either wx, pyq4, pyqt5 or pyside.
    If ever the environment variable PYTHONOCC_OFFSCREEN_RENDERER, then the GUI is simply
    ignored and an offscreen renderer is returned.
    init_display returns 4 objects :
    * display : an instance of Viewer3d ;
    * start_display : a function (the GUI mainloop) ;
    * add_menu : a function that creates a menu in the GUI
    * add_function_to_menu : adds a menu option

    In case an offscreen renderer is returned, start_display and add_menu are ignored, i.e.
    an empty function is returned (named do_nothing). add_function_to_menu just execute the
    function taken as a paramter.

    Note : the offscreen renderer is used on the travis side.
    """
    if size is None:  # prevent size to being None (mypy)
        raise AssertionError("window size cannot be None")

    if os.getenv("PYTHONOCC_OFFSCREEN_RENDERER") == "1":
        # create the offscreen renderer
        offscreen_renderer = OffscreenRenderer()

        def do_nothing(*kargs: Any, **kwargs: Any) -> None:
            """ takes as many parameters as you want,
            ans does nothing
            """
            pass

        def call_function(s, func: Callable) -> None:
            """ A function that calls another function.
            Helpfull to bypass add_function_to_menu. s should be a string
            """
            check_callable(func)
            log.info("Execute %s :: %s menu fonction" % (s, func.__name__))
            func()
            log.info("done")

        # returns empty classes and functions
        return offscreen_renderer, do_nothing, do_nothing, call_function
    used_backend = load_backend(backend_str)
    log.info("GUI backend set to: %s", used_backend)
    # wxPython based simple GUI
    if used_backend == 'wx':
        import wx
        from OCC.Display.wxDisplay import wxViewer3d

        class AppFrame(wx.Frame):

            def __init__(self, parent):
                wx.Frame.__init__(self, parent, -1, "pythonOCC-%s 3d viewer ('wx' backend)" % VERSION,
                                  style=wx.DEFAULT_FRAME_STYLE, size=size)
                self.canva = wxViewer3d(self)
                self.menuBar = wx.MenuBar()
                self._menus = {}
                self._menu_methods = {}

            def add_menu(self, menu_name: str) -> None:
                _menu = wx.Menu()
                self.menuBar.Append(_menu, "&" + menu_name)
                self.SetMenuBar(self.menuBar)
                self._menus[menu_name] = _menu

            def add_function_to_menu(self, menu_name: str, _callable: Callable) -> None:
                # point on curve
                _id = wx.NewId()
                check_callable(_callable)
                try:
                    self._menus[menu_name].Append(_id,
                                                  _callable.__name__.replace('_', ' ').lower())
                except KeyError:
                    raise ValueError('the menu item %s does not exist' % menu_name)
                self.Bind(wx.EVT_MENU, _callable, id=_id)

        app = wx.App(False)
        win = AppFrame(None)
        win.Show(True)
        wx.SafeYield()
        win.canva.InitDriver()
        app.SetTopWindow(win)
        display = win.canva._display

        def add_menu(*args, **kwargs) -> None:
            win.add_menu(*args, **kwargs)

        def add_function_to_menu(*args, **kwargs) -> None:
            win.add_function_to_menu(*args, **kwargs)

        def start_display() -> None:
            app.MainLoop()

    # Qt based simple GUI
    elif 'qt' in used_backend:
        from OCC.Display.qtDisplay import qtViewer3d
        QtCore, QtGui, QtWidgets, QtOpenGL = get_qt_modules()

        class MainWindow(QtWidgets.QMainWindow):

            def __init__(self, *args: Any) -> None:
                QtWidgets.QMainWindow.__init__(self, *args)
                self.canva = qtViewer3d(self)
                self.setWindowTitle("pythonOCC-%s 3d viewer ('%s' backend)" % (VERSION, used_backend))
                self.setCentralWidget(self.canva)
                if sys.platform != 'darwin':
                    self.menu_bar = self.menuBar()
                else:
                    # create a parentless menubar
                    # see: http://stackoverflow.com/questions/11375176/qmenubar-and-qmenu-doesnt-show-in-mac-os-x?lq=1
                    # noticeable is that the menu ( alas ) is created in the
                    # topleft of the screen, just
                    # next to the apple icon
                    # still does ugly things like showing the "Python" menu in
                    # bold
                    self.menu_bar = QtWidgets.QMenuBar()
                self._menus = {}
                self._menu_methods = {}
                # place the window in the center of the screen, at half the
                # screen size
                self.centerOnScreen()

            def centerOnScreen(self) -> None:
                '''Centers the window on the screen.'''
                resolution = QtWidgets.QApplication.desktop().screenGeometry()
                x = (resolution.width() - self.frameSize().width()) / 2
                y = (resolution.height() - self.frameSize().height()) / 2
                self.move(x, y)

            def add_menu(self, menu_name: str) -> None:
                _menu = self.menu_bar.addMenu("&" + menu_name)
                self._menus[menu_name] = _menu

            def add_function_to_menu(self, menu_name: str, _callable: Callable) -> None:
                check_callable(_callable)
                try:
                    _action = QtWidgets.QAction(_callable.__name__.replace('_', ' ').lower(), self)
                    # if not, the "exit" action is now shown...
                    _action.setMenuRole(QtWidgets.QAction.NoRole)
                    _action.triggered.connect(_callable)

                    self._menus[menu_name].addAction(_action)
                except KeyError:
                    raise ValueError('the menu item %s does not exist' % menu_name)

        # following couple of lines is a tweak to enable ipython --gui='qt'
        app = QtWidgets.QApplication.instance()  # checks if QApplication already exists
        if not app:  # create QApplication if it doesnt exist
            app = QtWidgets.QApplication(sys.argv)
        win = MainWindow()
        win.resize(size[0] -1, size[1] -1)
        win.show()
        win.centerOnScreen()
        win.canva.InitDriver()
        win.resize(size[0], size[1])
        win.canva.qApp = app
        display = win.canva._display

        def add_menu(*args, **kwargs) -> None:
            win.add_menu(*args, **kwargs)

        def add_function_to_menu(*args, **kwargs) -> None:
            win.add_function_to_menu(*args, **kwargs)

        def start_display() -> None:
            win.raise_()  # make the application float to the top
            app.exec_()

    if display_triedron:
        display.display_triedron()

    if background_gradient_color1 and background_gradient_color2:
    # background gradient
        display.set_bg_gradient_color(background_gradient_color1, background_gradient_color2)

    return display, start_display, add_menu, add_function_to_menu



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Adaptor3d import *
from OCC.Core.TopoDS import *
from OCC.Core.TopTools import *


class BRepCheck_ListOfStatus:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> BRepCheck_Status: ...
    def Last(self) -> BRepCheck_Status: ...
    def Append(self, theItem: BRepCheck_Status) -> BRepCheck_Status: ...
    def Prepend(self, theItem: BRepCheck_Status) -> BRepCheck_Status: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> BRepCheck_Status: ...
    def SetValue(self, theIndex: int, theValue: BRepCheck_Status) -> None: ...

class BRepCheck_Status(IntEnum):
	BRepCheck_NoError: int = ...
	BRepCheck_InvalidPointOnCurve: int = ...
	BRepCheck_InvalidPointOnCurveOnSurface: int = ...
	BRepCheck_InvalidPointOnSurface: int = ...
	BRepCheck_No3DCurve: int = ...
	BRepCheck_Multiple3DCurve: int = ...
	BRepCheck_Invalid3DCurve: int = ...
	BRepCheck_NoCurveOnSurface: int = ...
	BRepCheck_InvalidCurveOnSurface: int = ...
	BRepCheck_InvalidCurveOnClosedSurface: int = ...
	BRepCheck_InvalidSameRangeFlag: int = ...
	BRepCheck_InvalidSameParameterFlag: int = ...
	BRepCheck_InvalidDegeneratedFlag: int = ...
	BRepCheck_FreeEdge: int = ...
	BRepCheck_InvalidMultiConnexity: int = ...
	BRepCheck_InvalidRange: int = ...
	BRepCheck_EmptyWire: int = ...
	BRepCheck_RedundantEdge: int = ...
	BRepCheck_SelfIntersectingWire: int = ...
	BRepCheck_NoSurface: int = ...
	BRepCheck_InvalidWire: int = ...
	BRepCheck_RedundantWire: int = ...
	BRepCheck_IntersectingWires: int = ...
	BRepCheck_InvalidImbricationOfWires: int = ...
	BRepCheck_EmptyShell: int = ...
	BRepCheck_RedundantFace: int = ...
	BRepCheck_InvalidImbricationOfShells: int = ...
	BRepCheck_UnorientableShape: int = ...
	BRepCheck_NotClosed: int = ...
	BRepCheck_NotConnected: int = ...
	BRepCheck_SubshapeNotInShape: int = ...
	BRepCheck_BadOrientation: int = ...
	BRepCheck_BadOrientationOfSubshape: int = ...
	BRepCheck_InvalidPolygonOnTriangulation: int = ...
	BRepCheck_InvalidToleranceValue: int = ...
	BRepCheck_EnclosedRegion: int = ...
	BRepCheck_CheckFail: int = ...
BRepCheck_NoError = BRepCheck_Status.BRepCheck_NoError
BRepCheck_InvalidPointOnCurve = BRepCheck_Status.BRepCheck_InvalidPointOnCurve
BRepCheck_InvalidPointOnCurveOnSurface = BRepCheck_Status.BRepCheck_InvalidPointOnCurveOnSurface
BRepCheck_InvalidPointOnSurface = BRepCheck_Status.BRepCheck_InvalidPointOnSurface
BRepCheck_No3DCurve = BRepCheck_Status.BRepCheck_No3DCurve
BRepCheck_Multiple3DCurve = BRepCheck_Status.BRepCheck_Multiple3DCurve
BRepCheck_Invalid3DCurve = BRepCheck_Status.BRepCheck_Invalid3DCurve
BRepCheck_NoCurveOnSurface = BRepCheck_Status.BRepCheck_NoCurveOnSurface
BRepCheck_InvalidCurveOnSurface = BRepCheck_Status.BRepCheck_InvalidCurveOnSurface
BRepCheck_InvalidCurveOnClosedSurface = BRepCheck_Status.BRepCheck_InvalidCurveOnClosedSurface
BRepCheck_InvalidSameRangeFlag = BRepCheck_Status.BRepCheck_InvalidSameRangeFlag
BRepCheck_InvalidSameParameterFlag = BRepCheck_Status.BRepCheck_InvalidSameParameterFlag
BRepCheck_InvalidDegeneratedFlag = BRepCheck_Status.BRepCheck_InvalidDegeneratedFlag
BRepCheck_FreeEdge = BRepCheck_Status.BRepCheck_FreeEdge
BRepCheck_InvalidMultiConnexity = BRepCheck_Status.BRepCheck_InvalidMultiConnexity
BRepCheck_InvalidRange = BRepCheck_Status.BRepCheck_InvalidRange
BRepCheck_EmptyWire = BRepCheck_Status.BRepCheck_EmptyWire
BRepCheck_RedundantEdge = BRepCheck_Status.BRepCheck_RedundantEdge
BRepCheck_SelfIntersectingWire = BRepCheck_Status.BRepCheck_SelfIntersectingWire
BRepCheck_NoSurface = BRepCheck_Status.BRepCheck_NoSurface
BRepCheck_InvalidWire = BRepCheck_Status.BRepCheck_InvalidWire
BRepCheck_RedundantWire = BRepCheck_Status.BRepCheck_RedundantWire
BRepCheck_IntersectingWires = BRepCheck_Status.BRepCheck_IntersectingWires
BRepCheck_InvalidImbricationOfWires = BRepCheck_Status.BRepCheck_InvalidImbricationOfWires
BRepCheck_EmptyShell = BRepCheck_Status.BRepCheck_EmptyShell
BRepCheck_RedundantFace = BRepCheck_Status.BRepCheck_RedundantFace
BRepCheck_InvalidImbricationOfShells = BRepCheck_Status.BRepCheck_InvalidImbricationOfShells
BRepCheck_UnorientableShape = BRepCheck_Status.BRepCheck_UnorientableShape
BRepCheck_NotClosed = BRepCheck_Status.BRepCheck_NotClosed
BRepCheck_NotConnected = BRepCheck_Status.BRepCheck_NotConnected
BRepCheck_SubshapeNotInShape = BRepCheck_Status.BRepCheck_SubshapeNotInShape
BRepCheck_BadOrientation = BRepCheck_Status.BRepCheck_BadOrientation
BRepCheck_BadOrientationOfSubshape = BRepCheck_Status.BRepCheck_BadOrientationOfSubshape
BRepCheck_InvalidPolygonOnTriangulation = BRepCheck_Status.BRepCheck_InvalidPolygonOnTriangulation
BRepCheck_InvalidToleranceValue = BRepCheck_Status.BRepCheck_InvalidToleranceValue
BRepCheck_EnclosedRegion = BRepCheck_Status.BRepCheck_EnclosedRegion
BRepCheck_CheckFail = BRepCheck_Status.BRepCheck_CheckFail

class brepcheck:
	@staticmethod
	def Add(List: BRepCheck_ListOfStatus, Stat: BRepCheck_Status) -> None: ...
	@staticmethod
	def PrecCurve(aAC3D: Adaptor3d_Curve) -> float: ...
	@staticmethod
	def PrecSurface(aAHSurf: Adaptor3d_HSurface) -> float: ...
	@staticmethod
	def SelfIntersection(W: TopoDS_Wire, F: TopoDS_Face, E1: TopoDS_Edge, E2: TopoDS_Edge) -> bool: ...

class BRepCheck_Analyzer:
	def __init__(self, S: TopoDS_Shape, GeomControls: Optional[bool] = True) -> None: ...
	def Init(self, S: TopoDS_Shape, GeomControls: Optional[bool] = True) -> None: ...
	@overload
	def IsValid(self, S: TopoDS_Shape) -> bool: ...
	@overload
	def IsValid(self) -> bool: ...
	def Result(self, SubS: TopoDS_Shape) -> BRepCheck_Result: ...

class BRepCheck_Result(Standard_Transient):
	def Blind(self) -> None: ...
	def ContextualShape(self) -> TopoDS_Shape: ...
	def InContext(self, ContextShape: TopoDS_Shape) -> None: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def InitContextIterator(self) -> None: ...
	def IsBlind(self) -> bool: ...
	def IsMinimum(self) -> bool: ...
	def Minimum(self) -> None: ...
	def MoreShapeInContext(self) -> bool: ...
	def NextShapeInContext(self) -> None: ...
	def SetFailStatus(self, S: TopoDS_Shape) -> None: ...
	def Status(self) -> BRepCheck_ListOfStatus: ...
	@overload
	def StatusOnShape(self, S: TopoDS_Shape) -> BRepCheck_ListOfStatus: ...
	@overload
	def StatusOnShape(self) -> BRepCheck_ListOfStatus: ...

class BRepCheck_Edge(BRepCheck_Result):
	def __init__(self, E: TopoDS_Edge) -> None: ...
	def Blind(self) -> None: ...
	def CheckPolygonOnTriangulation(self, theEdge: TopoDS_Edge) -> BRepCheck_Status: ...
	@overload
	def GeometricControls(self) -> bool: ...
	@overload
	def GeometricControls(self, B: bool) -> None: ...
	def InContext(self, ContextShape: TopoDS_Shape) -> None: ...
	def Minimum(self) -> None: ...
	def SetStatus(self, theStatus: BRepCheck_Status) -> None: ...
	def Tolerance(self) -> float: ...

class BRepCheck_Face(BRepCheck_Result):
	def __init__(self, F: TopoDS_Face) -> None: ...
	def Blind(self) -> None: ...
	def ClassifyWires(self, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	@overload
	def GeometricControls(self) -> bool: ...
	@overload
	def GeometricControls(self, B: bool) -> None: ...
	def InContext(self, ContextShape: TopoDS_Shape) -> None: ...
	def IntersectWires(self, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	def IsUnorientable(self) -> bool: ...
	def Minimum(self) -> None: ...
	def OrientationOfWires(self, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	def SetStatus(self, theStatus: BRepCheck_Status) -> None: ...
	def SetUnorientable(self) -> None: ...

class BRepCheck_Shell(BRepCheck_Result):
	def __init__(self, S: TopoDS_Shell) -> None: ...
	def Blind(self) -> None: ...
	def Closed(self, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	def InContext(self, ContextShape: TopoDS_Shape) -> None: ...
	def IsUnorientable(self) -> bool: ...
	def Minimum(self) -> None: ...
	def NbConnectedSet(self, theSets: TopTools_ListOfShape) -> int: ...
	def Orientation(self, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	def SetUnorientable(self) -> None: ...

class BRepCheck_Solid(BRepCheck_Result):
	def __init__(self, theS: TopoDS_Solid) -> None: ...
	def Blind(self) -> None: ...
	def InContext(self, theContextShape: TopoDS_Shape) -> None: ...
	def Minimum(self) -> None: ...

class BRepCheck_Vertex(BRepCheck_Result):
	def __init__(self, V: TopoDS_Vertex) -> None: ...
	def Blind(self) -> None: ...
	def InContext(self, ContextShape: TopoDS_Shape) -> None: ...
	def Minimum(self) -> None: ...
	def Tolerance(self) -> float: ...

class BRepCheck_Wire(BRepCheck_Result):
	def __init__(self, W: TopoDS_Wire) -> None: ...
	def Blind(self) -> None: ...
	def Closed(self, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	def Closed2d(self, F: TopoDS_Face, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	@overload
	def GeometricControls(self) -> bool: ...
	@overload
	def GeometricControls(self, B: bool) -> None: ...
	def InContext(self, ContextShape: TopoDS_Shape) -> None: ...
	def Minimum(self) -> None: ...
	def Orientation(self, F: TopoDS_Face, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	def SelfIntersect(self, F: TopoDS_Face, E1: TopoDS_Edge, E2: TopoDS_Edge, Update: Optional[bool] = False) -> BRepCheck_Status: ...
	def SetStatus(self, theStatus: BRepCheck_Status) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

brepcheck_Add = brepcheck.Add
brepcheck_PrecCurve = brepcheck.PrecCurve
brepcheck_PrecSurface = brepcheck.PrecSurface
brepcheck_Print = brepcheck.Print
brepcheck_SelfIntersection = brepcheck.SelfIntersection



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
TopoDS module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_topods.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _TopoDS
else:
    import _TopoDS

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _TopoDS.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _TopoDS.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopoDS.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopoDS.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _TopoDS.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _TopoDS.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _TopoDS.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _TopoDS.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _TopoDS.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _TopoDS.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopoDS.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _TopoDS.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _TopoDS.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopoDS.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopoDS.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopoDS.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _TopoDS.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _TopoDS:
_TopoDS.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _TopoDS.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp

from enum import IntEnum
from OCC.Core.Exception import *




def Handle_TopoDS_HShape_Create() -> "opencascade::handle< TopoDS_HShape >":
    return _TopoDS.Handle_TopoDS_HShape_Create()

def Handle_TopoDS_HShape_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_HShape >":
    return _TopoDS.Handle_TopoDS_HShape_DownCast(t)

def Handle_TopoDS_HShape_IsNull(t: "opencascade::handle< TopoDS_HShape > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_HShape_IsNull(t)

def Handle_TopoDS_TShape_Create() -> "opencascade::handle< TopoDS_TShape >":
    return _TopoDS.Handle_TopoDS_TShape_Create()

def Handle_TopoDS_TShape_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TShape >":
    return _TopoDS.Handle_TopoDS_TShape_DownCast(t)

def Handle_TopoDS_TShape_IsNull(t: "opencascade::handle< TopoDS_TShape > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TShape_IsNull(t)

def Handle_TopoDS_TCompSolid_Create() -> "opencascade::handle< TopoDS_TCompSolid >":
    return _TopoDS.Handle_TopoDS_TCompSolid_Create()

def Handle_TopoDS_TCompSolid_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TCompSolid >":
    return _TopoDS.Handle_TopoDS_TCompSolid_DownCast(t)

def Handle_TopoDS_TCompSolid_IsNull(t: "opencascade::handle< TopoDS_TCompSolid > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TCompSolid_IsNull(t)

def Handle_TopoDS_TCompound_Create() -> "opencascade::handle< TopoDS_TCompound >":
    return _TopoDS.Handle_TopoDS_TCompound_Create()

def Handle_TopoDS_TCompound_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TCompound >":
    return _TopoDS.Handle_TopoDS_TCompound_DownCast(t)

def Handle_TopoDS_TCompound_IsNull(t: "opencascade::handle< TopoDS_TCompound > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TCompound_IsNull(t)

def Handle_TopoDS_TEdge_Create() -> "opencascade::handle< TopoDS_TEdge >":
    return _TopoDS.Handle_TopoDS_TEdge_Create()

def Handle_TopoDS_TEdge_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TEdge >":
    return _TopoDS.Handle_TopoDS_TEdge_DownCast(t)

def Handle_TopoDS_TEdge_IsNull(t: "opencascade::handle< TopoDS_TEdge > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TEdge_IsNull(t)

def Handle_TopoDS_TFace_Create() -> "opencascade::handle< TopoDS_TFace >":
    return _TopoDS.Handle_TopoDS_TFace_Create()

def Handle_TopoDS_TFace_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TFace >":
    return _TopoDS.Handle_TopoDS_TFace_DownCast(t)

def Handle_TopoDS_TFace_IsNull(t: "opencascade::handle< TopoDS_TFace > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TFace_IsNull(t)

def Handle_TopoDS_TShell_Create() -> "opencascade::handle< TopoDS_TShell >":
    return _TopoDS.Handle_TopoDS_TShell_Create()

def Handle_TopoDS_TShell_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TShell >":
    return _TopoDS.Handle_TopoDS_TShell_DownCast(t)

def Handle_TopoDS_TShell_IsNull(t: "opencascade::handle< TopoDS_TShell > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TShell_IsNull(t)

def Handle_TopoDS_TSolid_Create() -> "opencascade::handle< TopoDS_TSolid >":
    return _TopoDS.Handle_TopoDS_TSolid_Create()

def Handle_TopoDS_TSolid_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TSolid >":
    return _TopoDS.Handle_TopoDS_TSolid_DownCast(t)

def Handle_TopoDS_TSolid_IsNull(t: "opencascade::handle< TopoDS_TSolid > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TSolid_IsNull(t)

def Handle_TopoDS_TVertex_Create() -> "opencascade::handle< TopoDS_TVertex >":
    return _TopoDS.Handle_TopoDS_TVertex_Create()

def Handle_TopoDS_TVertex_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TVertex >":
    return _TopoDS.Handle_TopoDS_TVertex_DownCast(t)

def Handle_TopoDS_TVertex_IsNull(t: "opencascade::handle< TopoDS_TVertex > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TVertex_IsNull(t)

def Handle_TopoDS_TWire_Create() -> "opencascade::handle< TopoDS_TWire >":
    return _TopoDS.Handle_TopoDS_TWire_Create()

def Handle_TopoDS_TWire_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< TopoDS_TWire >":
    return _TopoDS.Handle_TopoDS_TWire_DownCast(t)

def Handle_TopoDS_TWire_IsNull(t: "opencascade::handle< TopoDS_TWire > const &") -> "bool":
    return _TopoDS.Handle_TopoDS_TWire_IsNull(t)
class TopoDS_ListIteratorOfListOfShape(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _TopoDS.TopoDS_ListIteratorOfListOfShape_swiginit(self, _TopoDS.new_TopoDS_ListIteratorOfListOfShape(*args))

    def More(self) -> "Standard_Boolean":
        return _TopoDS.TopoDS_ListIteratorOfListOfShape_More(self)

    def Next(self) -> "void":
        return _TopoDS.TopoDS_ListIteratorOfListOfShape_Next(self)

    def Value(self) -> "TopoDS_Shape const &":
        return _TopoDS.TopoDS_ListIteratorOfListOfShape_Value(self)

    def ChangeValue(self) -> "TopoDS_Shape &":
        return _TopoDS.TopoDS_ListIteratorOfListOfShape_ChangeValue(self)
    __swig_destroy__ = _TopoDS.delete_TopoDS_ListIteratorOfListOfShape

# Register TopoDS_ListIteratorOfListOfShape in _TopoDS:
_TopoDS.TopoDS_ListIteratorOfListOfShape_swigregister(TopoDS_ListIteratorOfListOfShape)

class TopoDS_ListOfShape(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_List< TopoDS_Shape >::iterator":
        return _TopoDS.TopoDS_ListOfShape_begin(self)

    def end(self) -> "NCollection_List< TopoDS_Shape >::iterator":
        return _TopoDS.TopoDS_ListOfShape_end(self)

    def cbegin(self) -> "NCollection_List< TopoDS_Shape >::const_iterator":
        return _TopoDS.TopoDS_ListOfShape_cbegin(self)

    def cend(self) -> "NCollection_List< TopoDS_Shape >::const_iterator":
        return _TopoDS.TopoDS_ListOfShape_cend(self)

    def __init__(self, *args):
        _TopoDS.TopoDS_ListOfShape_swiginit(self, _TopoDS.new_TopoDS_ListOfShape(*args))

    def Size(self) -> "Standard_Integer":
        return _TopoDS.TopoDS_ListOfShape_Size(self)

    def Assign(self, theOther: "TopoDS_ListOfShape") -> "NCollection_List< TopoDS_Shape > &":
        return _TopoDS.TopoDS_ListOfShape_Assign(self, theOther)

    def Set(self, theOther: "TopoDS_ListOfShape") -> "NCollection_List< TopoDS_Shape > &":
        return _TopoDS.TopoDS_ListOfShape_Set(self, theOther)

    def Clear(self, theAllocator: "opencascade::handle< NCollection_BaseAllocator > const &"=0) -> "void":
        return _TopoDS.TopoDS_ListOfShape_Clear(self, theAllocator)

    def First(self) -> "TopoDS_Shape const &":
        return _TopoDS.TopoDS_ListOfShape_First(self)

    def Last(self) -> "TopoDS_Shape const &":
        return _TopoDS.TopoDS_ListOfShape_Last(self)

    def Append(self, *args) -> "void":
        return _TopoDS.TopoDS_ListOfShape_Append(self, *args)

    def Prepend(self, *args) -> "void":
        return _TopoDS.TopoDS_ListOfShape_Prepend(self, *args)

    def RemoveFirst(self) -> "void":
        return _TopoDS.TopoDS_ListOfShape_RemoveFirst(self)

    def Remove(self, theIter: "TopoDS_ListIteratorOfListOfShape") -> "void":
        return _TopoDS.TopoDS_ListOfShape_Remove(self, theIter)

    def InsertBefore(self, *args) -> "void":
        return _TopoDS.TopoDS_ListOfShape_InsertBefore(self, *args)

    def InsertAfter(self, *args) -> "void":
        return _TopoDS.TopoDS_ListOfShape_InsertAfter(self, *args)

    def Reverse(self) -> "void":
        return _TopoDS.TopoDS_ListOfShape_Reverse(self)
    __swig_destroy__ = _TopoDS.delete_TopoDS_ListOfShape

    def __len__(self):
        return self.Size()


# Register TopoDS_ListOfShape in _TopoDS:
_TopoDS.TopoDS_ListOfShape_swigregister(TopoDS_ListOfShape)

class topods(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def CompSolid(*args) -> "TopoDS_CompSolid":
        r"""
        Casts shape s to the more specialized return type, compsolid. exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_CompSolid
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_CompSolid

        """
        return _TopoDS.topods_CompSolid(*args)

    @staticmethod
    def Compound(*args) -> "TopoDS_Compound":
        r"""
        Casts shape s to the more specialized return type, compound. exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Compound
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_Compound

        """
        return _TopoDS.topods_Compound(*args)

    @staticmethod
    def Edge(*args) -> "TopoDS_Edge":
        r"""
        Casts shape s to the more specialized return type, edge exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Edge
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_Edge

        """
        return _TopoDS.topods_Edge(*args)

    @staticmethod
    def Face(*args) -> "TopoDS_Face":
        r"""
        Casts shape s to the more specialized return type, face. exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Face
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_Face

        """
        return _TopoDS.topods_Face(*args)

    @staticmethod
    def Shell(*args) -> "TopoDS_Shell":
        r"""
        Casts shape s to the more specialized return type, shell. exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Shell
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_Shell

        """
        return _TopoDS.topods_Shell(*args)

    @staticmethod
    def Solid(*args) -> "TopoDS_Solid":
        r"""
        Casts shape s to the more specialized return type, solid. exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Solid
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_Solid

        """
        return _TopoDS.topods_Solid(*args)

    @staticmethod
    def Vertex(*args) -> "TopoDS_Vertex":
        r"""
        Basic tool to access the data structure. casts shape s to the more specialized return type, vertex. exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Vertex
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_Vertex

        """
        return _TopoDS.topods_Vertex(*args)

    @staticmethod
    def Wire(*args) -> "TopoDS_Wire":
        r"""
        Casts shape s to the more specialized return type, wire. exceptions standard_typemismatch if s cannot be cast to this return type.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        TopoDS_Wire
        No available documentation.

        Parameters
        ----------
        &: TopoDS_Shape

        Returns
        -------
        TopoDS_Wire

        """
        return _TopoDS.topods_Wire(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _TopoDS.topods_swiginit(self, _TopoDS.new_topods())
    __swig_destroy__ = _TopoDS.delete_topods

# Register topods in _TopoDS:
_TopoDS.topods_swigregister(topods)

def topods_CompSolid(*args) -> "TopoDS_CompSolid":
    r"""
    Casts shape s to the more specialized return type, compsolid. exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_CompSolid
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_CompSolid

    """
    return _TopoDS.topods_CompSolid(*args)

def topods_Compound(*args) -> "TopoDS_Compound":
    r"""
    Casts shape s to the more specialized return type, compound. exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_Compound
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_Compound

    """
    return _TopoDS.topods_Compound(*args)

def topods_Edge(*args) -> "TopoDS_Edge":
    r"""
    Casts shape s to the more specialized return type, edge exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_Edge
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_Edge

    """
    return _TopoDS.topods_Edge(*args)

def topods_Face(*args) -> "TopoDS_Face":
    r"""
    Casts shape s to the more specialized return type, face. exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_Face
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_Face

    """
    return _TopoDS.topods_Face(*args)

def topods_Shell(*args) -> "TopoDS_Shell":
    r"""
    Casts shape s to the more specialized return type, shell. exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_Shell
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_Shell

    """
    return _TopoDS.topods_Shell(*args)

def topods_Solid(*args) -> "TopoDS_Solid":
    r"""
    Casts shape s to the more specialized return type, solid. exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_Solid
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_Solid

    """
    return _TopoDS.topods_Solid(*args)

def topods_Vertex(*args) -> "TopoDS_Vertex":
    r"""
    Basic tool to access the data structure. casts shape s to the more specialized return type, vertex. exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_Vertex
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_Vertex

    """
    return _TopoDS.topods_Vertex(*args)

def topods_Wire(*args) -> "TopoDS_Wire":
    r"""
    Casts shape s to the more specialized return type, wire. exceptions standard_typemismatch if s cannot be cast to this return type.

    Parameters
    ----------
    S: TopoDS_Shape

    Returns
    -------
    TopoDS_Wire
    No available documentation.

    Parameters
    ----------
    &: TopoDS_Shape

    Returns
    -------
    TopoDS_Wire

    """
    return _TopoDS.topods_Wire(*args)

class TopoDS_AlertAttribute(OCC.Core.Message.Message_AttributeStream):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructor with shape argument.

        Parameters
        ----------
        theShape: TopoDS_Shape
        theName: TCollection_AsciiString,optional
        	default value is TCollection_AsciiString()

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_AlertAttribute_swiginit(self, _TopoDS.new_TopoDS_AlertAttribute(*args))

    def DumpJsonToString(self, depth: "int"=-1) -> "std::string":
        r"""DumpJsonToString(TopoDS_AlertAttribute self, int depth=-1) -> std::string"""
        return _TopoDS.TopoDS_AlertAttribute_DumpJsonToString(self, depth)

    def GetShape(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns contained shape.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_AlertAttribute_GetShape(self, *args)

    @staticmethod
    def Send(*args) -> "void":
        r"""
        Push shape information into messenger.

        Parameters
        ----------
        theMessenger: Message_Messenger
        theShape: TopoDS_Shape

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_AlertAttribute_Send(*args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_AlertAttribute

# Register TopoDS_AlertAttribute in _TopoDS:
_TopoDS.TopoDS_AlertAttribute_swigregister(TopoDS_AlertAttribute)

def TopoDS_AlertAttribute_Send(*args) -> "void":
    r"""
    Push shape information into messenger.

    Parameters
    ----------
    theMessenger: Message_Messenger
    theShape: TopoDS_Shape

    Returns
    -------
    None

    """
    return _TopoDS.TopoDS_AlertAttribute_Send(*args)

class TopoDS_AlertWithShape(OCC.Core.Message.Message_Alert):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructor with shape argument.

        Parameters
        ----------
        theShape: TopoDS_Shape

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_AlertWithShape_swiginit(self, _TopoDS.new_TopoDS_AlertWithShape(*args))

    def GetShape(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns contained shape.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_AlertWithShape_GetShape(self, *args)

    def SetShape(self, *args) -> "void":
        r"""
        Sets the shape.

        Parameters
        ----------
        theShape: TopoDS_Shape

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_AlertWithShape_SetShape(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_AlertWithShape

# Register TopoDS_AlertWithShape in _TopoDS:
_TopoDS.TopoDS_AlertWithShape_swigregister(TopoDS_AlertWithShape)

class TopoDS_Builder(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Add(self, *args) -> "void":
        r"""
        Add the shape c in the shape s. exceptions - topods_frozenshape if s is not free and cannot be modified. - topods__uncompatibleshapes if s and c are not compatible.

        Parameters
        ----------
        S: TopoDS_Shape
        C: TopoDS_Shape

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Builder_Add(self, *args)

    def MakeCompSolid(self, *args) -> "void":
        r"""
        Make an empty composite solid.

        Parameters
        ----------
        C: TopoDS_CompSolid

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Builder_MakeCompSolid(self, *args)

    def MakeCompound(self, *args) -> "void":
        r"""
        Make an empty compound.

        Parameters
        ----------
        C: TopoDS_Compound

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Builder_MakeCompound(self, *args)

    def MakeShell(self, *args) -> "void":
        r"""
        Make an empty shell.

        Parameters
        ----------
        S: TopoDS_Shell

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Builder_MakeShell(self, *args)

    def MakeSolid(self, *args) -> "void":
        r"""
        Make a solid covering the whole 3d space.

        Parameters
        ----------
        S: TopoDS_Solid

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Builder_MakeSolid(self, *args)

    def MakeWire(self, *args) -> "void":
        r"""
        Make an empty wire.

        Parameters
        ----------
        W: TopoDS_Wire

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Builder_MakeWire(self, *args)

    def Remove(self, *args) -> "void":
        r"""
        Remove the shape c from the shape s. exceptions topods_frozenshape if s is frozen and cannot be modified.

        Parameters
        ----------
        S: TopoDS_Shape
        C: TopoDS_Shape

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Builder_Remove(self, *args)

    __repr__ = _dumps_object


    def __init__(self):
        _TopoDS.TopoDS_Builder_swiginit(self, _TopoDS.new_TopoDS_Builder())
    __swig_destroy__ = _TopoDS.delete_TopoDS_Builder

# Register TopoDS_Builder in _TopoDS:
_TopoDS.TopoDS_Builder_swigregister(TopoDS_Builder)

class TopoDS_HShape(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructs an empty shape object.

        Returns
        -------
        None
        Constructs a shape object defined by the shape ashape.

        Parameters
        ----------
        aShape: TopoDS_Shape

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_HShape_swiginit(self, _TopoDS.new_TopoDS_HShape(*args))

    def ChangeShape(self, *args) -> "TopoDS_Shape":
        r"""
        Exchanges the topods_shape object defining this shape for another one referencing the same underlying shape accesses the list of shapes within the underlying shape referenced by the topods_shape object. returns a reference to a topods_shape based on this shape. the topods_shape can be modified.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_HShape_ChangeShape(self, *args)

    def Shape(self, *args) -> "TopoDS_Shape const":
        r"""
        Loads this shape with the shape ashape.

        Parameters
        ----------
        aShape: TopoDS_Shape

        Returns
        -------
        None
        Returns a reference to a constant topods_shape based on this shape.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_HShape_Shape(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_HShape_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_HShape

# Register TopoDS_HShape in _TopoDS:
_TopoDS.TopoDS_HShape_swigregister(TopoDS_HShape)

class TopoDS_Iterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty iterator.

        Returns
        -------
        None
        Creates an iterator on <s> sub-shapes. note: - if cumori is true, the function composes all sub-shapes with the orientation of s. - if cumloc is true, the function multiplies all sub-shapes by the location of s, i.e. it applies to each sub-shape the transformation that is associated with s.

        Parameters
        ----------
        S: TopoDS_Shape
        cumOri: bool,optional
        	default value is Standard_True
        cumLoc: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Iterator_swiginit(self, _TopoDS.new_TopoDS_Iterator(*args))

    def Initialize(self, *args) -> "void":
        r"""
        Initializes this iterator with shape s. note: - if cumori is true, the function composes all sub-shapes with the orientation of s. - if cumloc is true, the function multiplies all sub-shapes by the location of s, i.e. it applies to each sub-shape the transformation that is associated with s.

        Parameters
        ----------
        S: TopoDS_Shape
        cumOri: bool,optional
        	default value is Standard_True
        cumLoc: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Iterator_Initialize(self, *args)

    def More(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if there is another sub-shape in the shape which this iterator is scanning.

        Returns
        -------
        bool

        """
        return _TopoDS.TopoDS_Iterator_More(self, *args)

    def Next(self, *args) -> "void":
        r"""
        Moves on to the next sub-shape in the shape which this iterator is scanning. exceptions standard_nomoreobject if there are no more sub-shapes in the shape.

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Iterator_Next(self, *args)

    def Value(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the current sub-shape in the shape which this iterator is scanning. exceptions standard_nosuchobject if there is no current sub-shape.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Iterator_Value(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Iterator

# Register TopoDS_Iterator in _TopoDS:
_TopoDS.TopoDS_Iterator_swigregister(TopoDS_Iterator)

class TopoDS_Shape(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(TopoDS_Shape self, TopoDS_Shape arg0) -> TopoDS_ShapeCreates a null shape referring to nothing.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Shape_swiginit(self, _TopoDS.new_TopoDS_Shape(*args))

    def Checked(self, *args) -> "void":
        r"""
        Returns the checked flag.

        Returns
        -------
        bool
        Sets the checked flag.

        Parameters
        ----------
        theIsChecked: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Checked(self, *args)

    def Closed(self, *args) -> "void":
        r"""
        Returns the closedness flag.

        Returns
        -------
        bool
        Sets the closedness flag.

        Parameters
        ----------
        theIsClosed: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Closed(self, *args)

    def Complement(self, *args) -> "void":
        r"""
        Complements the orientation, using the complement method from the topabs package.

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Complement(self, *args)

    def Complemented(self, *args) -> "TopoDS_Shape":
        r"""
        Returns a shape similar to <self> with the orientation complemented, using the complement method from the topabs package.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Shape_Complemented(self, *args)

    def Compose(self, *args) -> "void":
        r"""
        Updates the shape orientation by composition with theorient, using the compose method from the topabs package.

        Parameters
        ----------
        theOrient: TopAbs_Orientation

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Compose(self, *args)

    def Composed(self, *args) -> "TopoDS_Shape":
        r"""
        Returns a shape similar to <self> with the orientation composed with theorient, using the compose method from the topabs package.

        Parameters
        ----------
        theOrient: TopAbs_Orientation

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Shape_Composed(self, *args)

    def Convex(self, *args) -> "void":
        r"""
        Returns the convexness flag.

        Returns
        -------
        bool
        Sets the convexness flag.

        Parameters
        ----------
        theIsConvex: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Convex(self, *args)

    def DumpJsonToString(self, depth: "int"=-1) -> "std::string":
        r"""DumpJsonToString(TopoDS_Shape self, int depth=-1) -> std::string"""
        return _TopoDS.TopoDS_Shape_DumpJsonToString(self, depth)

    def EmptyCopied(self, *args) -> "TopoDS_Shape":
        r"""
        Returns a new shape with the same orientation and location and a new tshape with the same geometry and no sub-shapes.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Shape_EmptyCopied(self, *args)

    def EmptyCopy(self, *args) -> "void":
        r"""
        Replace <self> by a new shape with the same orientation and location and a new tshape with the same geometry and no sub-shapes.

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_EmptyCopy(self, *args)

    def Free(self, *args) -> "void":
        r"""
        Returns the free flag.

        Returns
        -------
        bool
        Sets the free flag.

        Parameters
        ----------
        theIsFree: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Free(self, *args)

    def HashCode(self, *args) -> "Standard_Integer":
        r"""
        Returns a hashed value denoting <self>. this value is in the range [1, theupperbound]. it is computed from the tshape and the location. the orientation is not used. @param theupperbound the upper bound of the range a computing hash code must be within returns a computed hash code, in the range [1, theupperbound].

        Parameters
        ----------
        theUpperBound: int

        Returns
        -------
        int

        """
        return _TopoDS.TopoDS_Shape_HashCode(self, *args)

    def __hash__(self) -> "Standard_Integer":
        r"""__hash__(TopoDS_Shape self) -> Standard_Integer"""
        return _TopoDS.TopoDS_Shape___hash__(self)

    def Infinite(self, *args) -> "void":
        r"""
        Returns the infinity flag.

        Returns
        -------
        bool
        Sets the infinity flag.

        Parameters
        ----------
        theIsInfinite: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Infinite(self, *args)

    def IsEqual(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if two shapes are equal, i.e. if they share the same tshape with the same locations and orientations.

        Parameters
        ----------
        theOther: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _TopoDS.TopoDS_Shape_IsEqual(self, *args)

    def IsNotEqual(self, *args) -> "Standard_Boolean":
        r"""
        Negation of the isequal method.

        Parameters
        ----------
        theOther: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _TopoDS.TopoDS_Shape_IsNotEqual(self, *args)

    def IsNull(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if this shape is null. in other words, it references no underlying shape with the potential to be given a location and an orientation.

        Returns
        -------
        bool

        """
        return _TopoDS.TopoDS_Shape_IsNull(self, *args)

    def IsPartner(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if two shapes are partners, i.e. if they share the same tshape. locations and orientations may differ.

        Parameters
        ----------
        theOther: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _TopoDS.TopoDS_Shape_IsPartner(self, *args)

    def IsSame(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if two shapes are same, i.e. if they share the same tshape with the same locations. orientations may differ.

        Parameters
        ----------
        theOther: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _TopoDS.TopoDS_Shape_IsSame(self, *args)

    def Located(self, *args) -> "TopoDS_Shape":
        r"""
        Returns a shape similar to <self> with the local coordinate system set to <loc>.

        Parameters
        ----------
        theLoc: TopLoc_Location

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Shape_Located(self, *args)

    def Location(self, *args) -> "void":
        r"""
        Returns the shape local coordinate system.

        Returns
        -------
        TopLoc_Location
        Sets the shape local coordinate system.

        Parameters
        ----------
        theLoc: TopLoc_Location

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Location(self, *args)

    def Locked(self, *args) -> "void":
        r"""
        Returns the locked flag.

        Returns
        -------
        bool
        Sets the locked flag.

        Parameters
        ----------
        theIsLocked: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Locked(self, *args)

    def Modified(self, *args) -> "void":
        r"""
        Returns the modification flag.

        Returns
        -------
        bool
        Sets the modification flag.

        Parameters
        ----------
        theIsModified: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Modified(self, *args)

    def Move(self, *args) -> "void":
        r"""
        Multiplies the shape location by theposition.

        Parameters
        ----------
        thePosition: TopLoc_Location

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Move(self, *args)

    def Moved(self, *args) -> "TopoDS_Shape":
        r"""
        Returns a shape similar to <self> with a location multiplied by theposition.

        Parameters
        ----------
        thePosition: TopLoc_Location

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Shape_Moved(self, *args)

    def NbChildren(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of direct sub-shapes (children). @sa topods_iterator for accessing sub-shapes.

        Returns
        -------
        int

        """
        return _TopoDS.TopoDS_Shape_NbChildren(self, *args)

    def Nullify(self, *args) -> "void":
        r"""
        Destroys the reference to the underlying shape stored in this shape. as a result, this shape becomes null.

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Nullify(self, *args)

    def Orientable(self, *args) -> "void":
        r"""
        Returns the orientability flag.

        Returns
        -------
        bool
        Sets the orientability flag.

        Parameters
        ----------
        theIsOrientable: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Orientable(self, *args)

    def Orientation(self, *args) -> "void":
        r"""
        Returns the shape orientation.

        Returns
        -------
        TopAbs_Orientation
        Sets the shape orientation.

        Parameters
        ----------
        theOrient: TopAbs_Orientation

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Orientation(self, *args)

    def Oriented(self, *args) -> "TopoDS_Shape":
        r"""
        Returns a shape similar to <self> with the orientation set to <or>.

        Parameters
        ----------
        theOrient: TopAbs_Orientation

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Shape_Oriented(self, *args)

    def Reverse(self, *args) -> "void":
        r"""
        Reverses the orientation, using the reverse method from the topabs package.

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_Reverse(self, *args)

    def Reversed(self, *args) -> "TopoDS_Shape":
        r"""
        Returns a shape similar to <self> with the orientation reversed, using the reverse method from the topabs package.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopoDS.TopoDS_Shape_Reversed(self, *args)

    def ShapeType(self, *args) -> "TopAbs_ShapeEnum":
        r"""
        Returns the value of the topabs_shapeenum enumeration that corresponds to this shape, for example vertex, edge, and so on. exceptions standard_nullobject if this shape is null.

        Returns
        -------
        TopAbs_ShapeEnum

        """
        return _TopoDS.TopoDS_Shape_ShapeType(self, *args)

    def TShape(self, *args) -> "void":
        r"""
        Returns a handle to the actual shape implementation.

        Returns
        -------
        opencascade::handle<TopoDS_TShape>
        No available documentation.

        Parameters
        ----------
        theTShape: TopoDS_TShape

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_Shape_TShape(self, *args)

    def __ne_wrapper__(self, other: "TopoDS_Shape") -> "bool":
        r"""__ne_wrapper__(TopoDS_Shape self, TopoDS_Shape other) -> bool"""
        return _TopoDS.TopoDS_Shape___ne_wrapper__(self, other)

    def __ne__(self, right):
        try:
            return self.__ne_wrapper__(right)
        except:
            return True


    def __eq_wrapper__(self, other: "TopoDS_Shape") -> "bool":
        r"""__eq_wrapper__(TopoDS_Shape self, TopoDS_Shape other) -> bool"""
        return _TopoDS.TopoDS_Shape___eq_wrapper__(self, other)

    def __eq__(self, right):
        try:
            return self.__eq_wrapper__(right)
        except:
            return False


    def __getstate__(self):
    	from .BRepTools import BRepTools_ShapeSet
    	ss = BRepTools_ShapeSet()
    	ss.Add(self)
    	str_shape = ss.WriteToString()
    	indx = ss.Locations().Index(self.Location())
    	return str_shape, indx
    def __setstate__(self, state):
    	from .BRepTools import BRepTools_ShapeSet
    	topods_str, indx = state
    	ss = BRepTools_ShapeSet()
    	ss.ReadFromString(topods_str)
    	the_shape = ss.Shape(ss.NbShapes())
    	location = ss.Locations().Location(indx)
    	the_shape.Location(location)
    	self.this = the_shape.this


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Shape

# Register TopoDS_Shape in _TopoDS:
_TopoDS.TopoDS_Shape_swigregister(TopoDS_Shape)

class TopoDS_TShape(OCC.Core.Standard.Standard_Transient):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def Checked(self, *args) -> "void":
        r"""
        Returns the checked flag.

        Returns
        -------
        bool
        Sets the checked flag.

        Parameters
        ----------
        theIsChecked: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Checked(self, *args)

    def Closed(self, *args) -> "void":
        r"""
        Returns the closedness flag.

        Returns
        -------
        bool
        Sets the closedness flag.

        Parameters
        ----------
        theIsClosed: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Closed(self, *args)

    def Convex(self, *args) -> "void":
        r"""
        Returns the convexness flag.

        Returns
        -------
        bool
        Sets the convexness flag.

        Parameters
        ----------
        theIsConvex: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Convex(self, *args)

    def DumpJsonToString(self, depth: "int"=-1) -> "std::string":
        r"""DumpJsonToString(TopoDS_TShape self, int depth=-1) -> std::string"""
        return _TopoDS.TopoDS_TShape_DumpJsonToString(self, depth)

    def EmptyCopy(self, *args) -> "opencascade::handle< TopoDS_TShape >":
        r"""
        Returns a copy of the tshape with no sub-shapes.

        Returns
        -------
        opencascade::handle<TopoDS_TShape>

        """
        return _TopoDS.TopoDS_TShape_EmptyCopy(self, *args)

    def Free(self, *args) -> "void":
        r"""
        Returns the free flag.

        Returns
        -------
        bool
        Sets the free flag.

        Parameters
        ----------
        theIsFree: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Free(self, *args)

    def Infinite(self, *args) -> "void":
        r"""
        Returns the infinity flag.

        Returns
        -------
        bool
        Sets the infinity flag.

        Parameters
        ----------
        theIsInfinite: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Infinite(self, *args)

    def Locked(self, *args) -> "void":
        r"""
        Returns the locked flag.

        Returns
        -------
        bool
        Sets the locked flag.

        Parameters
        ----------
        theIsLocked: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Locked(self, *args)

    def Modified(self, *args) -> "void":
        r"""
        Returns the modification flag.

        Returns
        -------
        bool
        Sets the modification flag.

        Parameters
        ----------
        theIsModified: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Modified(self, *args)

    def NbChildren(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of direct sub-shapes (children). @sa topods_iterator for accessing sub-shapes.

        Returns
        -------
        int

        """
        return _TopoDS.TopoDS_TShape_NbChildren(self, *args)

    def Orientable(self, *args) -> "void":
        r"""
        Returns the orientability flag.

        Returns
        -------
        bool
        Sets the orientability flag.

        Parameters
        ----------
        theIsOrientable: bool

        Returns
        -------
        None

        """
        return _TopoDS.TopoDS_TShape_Orientable(self, *args)

    def ShapeType(self, *args) -> "TopAbs_ShapeEnum":
        r"""
        Returns the type as a term of the shapeenum enum : vertex, edge, wire, face, ....

        Returns
        -------
        TopAbs_ShapeEnum

        """
        return _TopoDS.TopoDS_TShape_ShapeType(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TShape_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TShape

# Register TopoDS_TShape in _TopoDS:
_TopoDS.TopoDS_TShape_swigregister(TopoDS_TShape)

class TopoDS_CompSolid(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructs an undefined compsolid.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_CompSolid_swiginit(self, _TopoDS.new_TopoDS_CompSolid(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_CompSolid

# Register TopoDS_CompSolid in _TopoDS:
_TopoDS.TopoDS_CompSolid_swigregister(TopoDS_CompSolid)

class TopoDS_Compound(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructs an undefined compound.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Compound_swiginit(self, _TopoDS.new_TopoDS_Compound(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Compound

# Register TopoDS_Compound in _TopoDS:
_TopoDS.TopoDS_Compound_swigregister(TopoDS_Compound)

class TopoDS_Edge(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Undefined edge.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Edge_swiginit(self, _TopoDS.new_TopoDS_Edge(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Edge

# Register TopoDS_Edge in _TopoDS:
_TopoDS.TopoDS_Edge_swigregister(TopoDS_Edge)

class TopoDS_Face(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Undefined face.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Face_swiginit(self, _TopoDS.new_TopoDS_Face(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Face

# Register TopoDS_Face in _TopoDS:
_TopoDS.TopoDS_Face_swigregister(TopoDS_Face)

class TopoDS_Shell(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructs an undefined shell.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Shell_swiginit(self, _TopoDS.new_TopoDS_Shell(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Shell

# Register TopoDS_Shell in _TopoDS:
_TopoDS.TopoDS_Shell_swigregister(TopoDS_Shell)

class TopoDS_Solid(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Constructs an undefined solid.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Solid_swiginit(self, _TopoDS.new_TopoDS_Solid(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Solid

# Register TopoDS_Solid in _TopoDS:
_TopoDS.TopoDS_Solid_swigregister(TopoDS_Solid)

class TopoDS_TCompSolid(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty tcompsolid.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_TCompSolid_swiginit(self, _TopoDS.new_TopoDS_TCompSolid(*args))


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TCompSolid_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TCompSolid

# Register TopoDS_TCompSolid in _TopoDS:
_TopoDS.TopoDS_TCompSolid_swigregister(TopoDS_TCompSolid)

class TopoDS_TCompound(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty tcompound.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_TCompound_swiginit(self, _TopoDS.new_TopoDS_TCompound(*args))


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TCompound_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TCompound

# Register TopoDS_TCompound in _TopoDS:
_TopoDS.TopoDS_TCompound_swigregister(TopoDS_TCompound)

class TopoDS_TEdge(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TEdge_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TEdge

# Register TopoDS_TEdge in _TopoDS:
_TopoDS.TopoDS_TEdge_swigregister(TopoDS_TEdge)

class TopoDS_TFace(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty tface.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_TFace_swiginit(self, _TopoDS.new_TopoDS_TFace(*args))


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TFace_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TFace

# Register TopoDS_TFace in _TopoDS:
_TopoDS.TopoDS_TFace_swigregister(TopoDS_TFace)

class TopoDS_TShell(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty tshell.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_TShell_swiginit(self, _TopoDS.new_TopoDS_TShell(*args))


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TShell_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TShell

# Register TopoDS_TShell in _TopoDS:
_TopoDS.TopoDS_TShell_swigregister(TopoDS_TShell)

class TopoDS_TSolid(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty tsolid.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_TSolid_swiginit(self, _TopoDS.new_TopoDS_TSolid(*args))


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TSolid_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TSolid

# Register TopoDS_TSolid in _TopoDS:
_TopoDS.TopoDS_TSolid_swigregister(TopoDS_TSolid)

class TopoDS_TVertex(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TVertex_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TVertex

# Register TopoDS_TVertex in _TopoDS:
_TopoDS.TopoDS_TVertex_swigregister(TopoDS_TVertex)

class TopoDS_TWire(TopoDS_TShape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty twire.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_TWire_swiginit(self, _TopoDS.new_TopoDS_TWire(*args))


    @staticmethod
    def DownCast(t):
      return Handle_TopoDS_TWire_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_TWire

# Register TopoDS_TWire in _TopoDS:
_TopoDS.TopoDS_TWire_swigregister(TopoDS_TWire)

class TopoDS_Vertex(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(TopoDS_Vertex self, TopoDS_Vertex arg0) -> TopoDS_VertexUndefined vertex.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Vertex_swiginit(self, _TopoDS.new_TopoDS_Vertex(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Vertex

# Register TopoDS_Vertex in _TopoDS:
_TopoDS.TopoDS_Vertex_swigregister(TopoDS_Vertex)

class TopoDS_Wire(TopoDS_Shape):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Undefined wire.

        Returns
        -------
        None

        """
        _TopoDS.TopoDS_Wire_swiginit(self, _TopoDS.new_TopoDS_Wire(*args))

    __repr__ = _dumps_object

    __swig_destroy__ = _TopoDS.delete_TopoDS_Wire

# Register TopoDS_Wire in _TopoDS:
_TopoDS.TopoDS_Wire_swigregister(TopoDS_Wire)








# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
TopExp module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_topexp.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _TopExp
else:
    import _TopExp

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _TopExp.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _TopExp.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopExp.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopExp.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _TopExp.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _TopExp.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _TopExp.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _TopExp.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _TopExp.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _TopExp.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopExp.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _TopExp.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _TopExp.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopExp.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopExp.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopExp.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _TopExp.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _TopExp:
_TopExp.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _TopExp.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.TopoDS
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp
import OCC.Core.TopTools

from enum import IntEnum
from OCC.Core.Exception import *



class topexp(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def CommonVertex(*args) -> "Standard_Boolean":
        r"""
        Finds the vertex <v> common to the two edges <e1,e2>, returns true if this vertex exists. //! warning: <v> has sense only if the value <true> is returned.

        Parameters
        ----------
        E1: TopoDS_Edge
        E2: TopoDS_Edge
        V: TopoDS_Vertex

        Returns
        -------
        bool

        """
        return _TopExp.topexp_CommonVertex(*args)

    @staticmethod
    def FirstVertex(*args) -> "TopoDS_Vertex":
        r"""
        Returns the vertex of orientation forward in e. if there is none returns a null shape. cumori = true : taking account the edge orientation.

        Parameters
        ----------
        E: TopoDS_Edge
        CumOri: bool,optional
        	default value is Standard_False

        Returns
        -------
        TopoDS_Vertex

        """
        return _TopExp.topexp_FirstVertex(*args)

    @staticmethod
    def LastVertex(*args) -> "TopoDS_Vertex":
        r"""
        Returns the vertex of orientation reversed in e. if there is none returns a null shape. cumori = true : taking account the edge orientation.

        Parameters
        ----------
        E: TopoDS_Edge
        CumOri: bool,optional
        	default value is Standard_False

        Returns
        -------
        TopoDS_Vertex

        """
        return _TopExp.topexp_LastVertex(*args)

    @staticmethod
    def MapShapes(*args) -> "void":
        r"""
        Tool to explore a topological data structure. stores in the map <m> all the sub-shapes of <s> of type <t>. //! warning: the map is not cleared at first.

        Parameters
        ----------
        S: TopoDS_Shape
        T: TopAbs_ShapeEnum
        M: TopTools_IndexedMapOfShape

        Returns
        -------
        None
        Stores in the map <m> all the sub-shapes of <s>.

        Parameters
        ----------
        S: TopoDS_Shape
        M: TopTools_IndexedMapOfShape

        Returns
        -------
        None
        Stores in the map <m> all the sub-shapes of <s>.

        Parameters
        ----------
        S: TopoDS_Shape
        M: TopTools_MapOfShape

        Returns
        -------
        None

        """
        return _TopExp.topexp_MapShapes(*args)

    @staticmethod
    def MapShapesAndAncestors(*args) -> "void":
        r"""
        Stores in the map <m> all the subshape of <s> of type <ts> for each one append to the list all the ancestors of type <ta>. for example map all the edges and bind the list of faces. warning: the map is not cleared at first.

        Parameters
        ----------
        S: TopoDS_Shape
        TS: TopAbs_ShapeEnum
        TA: TopAbs_ShapeEnum
        M: TopTools_IndexedDataMapOfShapeListOfShape

        Returns
        -------
        None

        """
        return _TopExp.topexp_MapShapesAndAncestors(*args)

    @staticmethod
    def MapShapesAndUniqueAncestors(*args) -> "void":
        r"""
        Stores in the map <m> all the subshape of <s> of type <ts> for each one append to the list all unique ancestors of type <ta>. for example map all the edges and bind the list of faces. useorientation = true : taking account the ancestor orientation warning: the map is not cleared at first.

        Parameters
        ----------
        S: TopoDS_Shape
        TS: TopAbs_ShapeEnum
        TA: TopAbs_ShapeEnum
        M: TopTools_IndexedDataMapOfShapeListOfShape
        useOrientation: bool,optional
        	default value is Standard_False

        Returns
        -------
        None

        """
        return _TopExp.topexp_MapShapesAndUniqueAncestors(*args)

    @staticmethod
    def Vertices(*args) -> "void":
        r"""
        Returns in vfirst, vlast the forward and reversed vertices of the edge <e>. may be null shapes. cumori = true : taking account the edge orientation.

        Parameters
        ----------
        E: TopoDS_Edge
        Vfirst: TopoDS_Vertex
        Vlast: TopoDS_Vertex
        CumOri: bool,optional
        	default value is Standard_False

        Returns
        -------
        None
        Returns in vfirst, vlast the first and last vertices of the open wire <w>. may be null shapes. if <w> is closed vfirst and vlast are a same vertex on <w>. if <w> is no manifold. vfirst and vlast are null shapes.

        Parameters
        ----------
        W: TopoDS_Wire
        Vfirst: TopoDS_Vertex
        Vlast: TopoDS_Vertex

        Returns
        -------
        None

        """
        return _TopExp.topexp_Vertices(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _TopExp.topexp_swiginit(self, _TopExp.new_topexp())
    __swig_destroy__ = _TopExp.delete_topexp

# Register topexp in _TopExp:
_TopExp.topexp_swigregister(topexp)

def topexp_CommonVertex(*args) -> "Standard_Boolean":
    r"""
    Finds the vertex <v> common to the two edges <e1,e2>, returns true if this vertex exists. //! warning: <v> has sense only if the value <true> is returned.

    Parameters
    ----------
    E1: TopoDS_Edge
    E2: TopoDS_Edge
    V: TopoDS_Vertex

    Returns
    -------
    bool

    """
    return _TopExp.topexp_CommonVertex(*args)

def topexp_FirstVertex(*args) -> "TopoDS_Vertex":
    r"""
    Returns the vertex of orientation forward in e. if there is none returns a null shape. cumori = true : taking account the edge orientation.

    Parameters
    ----------
    E: TopoDS_Edge
    CumOri: bool,optional
    	default value is Standard_False

    Returns
    -------
    TopoDS_Vertex

    """
    return _TopExp.topexp_FirstVertex(*args)

def topexp_LastVertex(*args) -> "TopoDS_Vertex":
    r"""
    Returns the vertex of orientation reversed in e. if there is none returns a null shape. cumori = true : taking account the edge orientation.

    Parameters
    ----------
    E: TopoDS_Edge
    CumOri: bool,optional
    	default value is Standard_False

    Returns
    -------
    TopoDS_Vertex

    """
    return _TopExp.topexp_LastVertex(*args)

def topexp_MapShapes(*args) -> "void":
    r"""
    Tool to explore a topological data structure. stores in the map <m> all the sub-shapes of <s> of type <t>. //! warning: the map is not cleared at first.

    Parameters
    ----------
    S: TopoDS_Shape
    T: TopAbs_ShapeEnum
    M: TopTools_IndexedMapOfShape

    Returns
    -------
    None
    Stores in the map <m> all the sub-shapes of <s>.

    Parameters
    ----------
    S: TopoDS_Shape
    M: TopTools_IndexedMapOfShape

    Returns
    -------
    None
    Stores in the map <m> all the sub-shapes of <s>.

    Parameters
    ----------
    S: TopoDS_Shape
    M: TopTools_MapOfShape

    Returns
    -------
    None

    """
    return _TopExp.topexp_MapShapes(*args)

def topexp_MapShapesAndAncestors(*args) -> "void":
    r"""
    Stores in the map <m> all the subshape of <s> of type <ts> for each one append to the list all the ancestors of type <ta>. for example map all the edges and bind the list of faces. warning: the map is not cleared at first.

    Parameters
    ----------
    S: TopoDS_Shape
    TS: TopAbs_ShapeEnum
    TA: TopAbs_ShapeEnum
    M: TopTools_IndexedDataMapOfShapeListOfShape

    Returns
    -------
    None

    """
    return _TopExp.topexp_MapShapesAndAncestors(*args)

def topexp_MapShapesAndUniqueAncestors(*args) -> "void":
    r"""
    Stores in the map <m> all the subshape of <s> of type <ts> for each one append to the list all unique ancestors of type <ta>. for example map all the edges and bind the list of faces. useorientation = true : taking account the ancestor orientation warning: the map is not cleared at first.

    Parameters
    ----------
    S: TopoDS_Shape
    TS: TopAbs_ShapeEnum
    TA: TopAbs_ShapeEnum
    M: TopTools_IndexedDataMapOfShapeListOfShape
    useOrientation: bool,optional
    	default value is Standard_False

    Returns
    -------
    None

    """
    return _TopExp.topexp_MapShapesAndUniqueAncestors(*args)

def topexp_Vertices(*args) -> "void":
    r"""
    Returns in vfirst, vlast the forward and reversed vertices of the edge <e>. may be null shapes. cumori = true : taking account the edge orientation.

    Parameters
    ----------
    E: TopoDS_Edge
    Vfirst: TopoDS_Vertex
    Vlast: TopoDS_Vertex
    CumOri: bool,optional
    	default value is Standard_False

    Returns
    -------
    None
    Returns in vfirst, vlast the first and last vertices of the open wire <w>. may be null shapes. if <w> is closed vfirst and vlast are a same vertex on <w>. if <w> is no manifold. vfirst and vlast are null shapes.

    Parameters
    ----------
    W: TopoDS_Wire
    Vfirst: TopoDS_Vertex
    Vlast: TopoDS_Vertex

    Returns
    -------
    None

    """
    return _TopExp.topexp_Vertices(*args)

class TopExp_Explorer(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty explorer, becomes usefull after init.

        Returns
        -------
        None
        Creates an explorer on the shape <s>. //! <tofind> is the type of shapes to search. topabs_vertex, topabs_edge, ... //! <toavoid> is the type of shape to skip in the exploration. if <toavoid> is equal or less complex than <tofind> or if <toavoid> is shape it has no effect on the exploration.

        Parameters
        ----------
        S: TopoDS_Shape
        ToFind: TopAbs_ShapeEnum
        ToAvoid: TopAbs_ShapeEnum,optional
        	default value is TopAbs_SHAPE

        Returns
        -------
        None

        """
        _TopExp.TopExp_Explorer_swiginit(self, _TopExp.new_TopExp_Explorer(*args))

    def Clear(self, *args) -> "void":
        r"""
        Clears the content of the explorer. it will return false on more().

        Returns
        -------
        None

        """
        return _TopExp.TopExp_Explorer_Clear(self, *args)

    def Current(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the current shape in the exploration. exceptions standard_nosuchobject if this explorer has no more shapes to explore.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopExp.TopExp_Explorer_Current(self, *args)

    def Depth(self, *args) -> "Standard_Integer":
        r"""
        Returns the current depth of the exploration. 0 is the shape to explore itself.

        Returns
        -------
        int

        """
        return _TopExp.TopExp_Explorer_Depth(self, *args)

    def Destroy(self, *args) -> "void":
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        return _TopExp.TopExp_Explorer_Destroy(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Resets this explorer on the shape s. it is initialized to search the shape s, for shapes of type tofind, that are not part of a shape toavoid. if the shape toavoid is equal to topabs_shape, or if it is the same as, or less complex than, the shape tofind it has no effect on the search.

        Parameters
        ----------
        S: TopoDS_Shape
        ToFind: TopAbs_ShapeEnum
        ToAvoid: TopAbs_ShapeEnum,optional
        	default value is TopAbs_SHAPE

        Returns
        -------
        None

        """
        return _TopExp.TopExp_Explorer_Init(self, *args)

    def More(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if there are more shapes in the exploration.

        Returns
        -------
        bool

        """
        return _TopExp.TopExp_Explorer_More(self, *args)

    def Next(self, *args) -> "void":
        r"""
        Moves to the next shape in the exploration. exceptions standard_nomoreobject if there are no more shapes to explore.

        Returns
        -------
        None

        """
        return _TopExp.TopExp_Explorer_Next(self, *args)

    def ReInit(self, *args) -> "void":
        r"""
        Reinitialize the exploration with the original arguments.

        Returns
        -------
        None

        """
        return _TopExp.TopExp_Explorer_ReInit(self, *args)

    def Value(self, *args) -> "TopoDS_Shape const":
        r"""
        Returns the current shape in the exploration. exceptions standard_nosuchobject if this explorer has no more shapes to explore.

        Returns
        -------
        TopoDS_Shape

        """
        return _TopExp.TopExp_Explorer_Value(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TopExp.delete_TopExp_Explorer

# Register TopExp_Explorer in _TopExp:
_TopExp.TopExp_Explorer_swigregister(TopExp_Explorer)





